<?php
/***********************************************************
 Copyright (C) 2013 Hewlett-Packard Development Company, L.P.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 ***********************************************************/

/**
 * @file fo_bucket_list.php
 *
 * @brief get a list of filepaths and bucket information for those
 * files. 
 *
 */

$Usage = "Usage: " . basename($argv[0]) . "
  -u upload id        :: upload id
  -t uploadtree id    :: uploadtree id
  -c sysconfdir       :: optional - Specify the directory for the system configuration
  --user username     :: user name
  --password password :: password
  -b bucket id        :: bucket id
  -a bucket agent id  :: bucket agent id
  -n nomos agent id   :: nomos agent id
  -h  help, this message
  ";

$upload = $item = $bucket = $bucket_agent = $nomos_agent = "";

$longopts = array("user:", "password:");
$options = getopt("c:u:t:b:a:n:h", $longopts);
if (empty($options) || !is_array($options))
{
  print $Usage;
  return 1;
}

$user = $passwd = "";

foreach($options as $option => $value)
{
  switch($option)
  {
    case 'c': // handled in fo_wrapper
      break;
    case 'u':
      $upload = $value;
      break;
    case 't':
      $item = $value;
      break;
    case 'a':
      $bucket_agent = $value;
      break;
    case 'n':
      $nomos_agent = $value;
      break;
    case 'b':
      $bucket = $value;
      break;
    case 'h':
      print $Usage;
      return 1;
    case 'user':
      $user = $value;
      break;
    case 'password':
      $passwd = $value;
      break;
    default:
      print "unknown option $option\n";
      print $Usage;
  }
}

if (empty($item) || empty($bucket) || empty($bucket_agent) || empty($nomos_agent)) {
  print "Uploadtree ID  or bucket ID or bucket agent ID or nomos agent ID is NULL\n";
  print $Usage;
  return 1;
}

/** get upload id through uploadtree id */
if (is_numeric($item) && !is_numeric($upload)) $upload = GetUploadID($item);

//print "Upload ID, Uploadtree ID, bucket ID, bucket agent ID, nomos agent ID is $upload, $item, $bucket, $bucket_agent, $nomos_agent\n";

/** check if parameters are valid */
if (!is_numeric($bucket) && (!is_numeric($upload) || (!empty($item) && !is_numeric($item))) 
    && !is_numeric($bucket_agent) && !is_numeric($nomos_agent))
{
  print "Upload ID or Uploadtree ID  or bucket ID or bucket agent ID or nomos agent ID is not digital number\n";
  print $Usage;
  return 1;
}

account_check($user, $passwd); // check username/password

$return_value = read_permission($upload, $user); // check if the user has the permission to read this upload
if (empty($return_value))
{
  $text = _("The user '$user' has no permission to read the information of upload $upload\n");
  echo $text;
  return 1;
}

require_once("$MODDIR/lib/php/common.php");

/** get bucket information for this uploadtree */
GetBucketList($bucket, $bucket_agent, $nomos_agent, $item, $upload);
return 0;

/**
 * \brief get bucket list of one specified upload or all uploads
 *
 * \pamam $upload_pk - upload id
 * \param $bucket_pk - bucket id
 * \param $bucket_agent - bucket agent ID
 * \param $nomos_agent - nomos agent ID
 * \prram $uploadtree_pk - uploadtree ID
 */
function GetBucketList($bucket_pk, $bucket_agent, $nomos_agent, $uploadtree_pk, $upload_pk = 0)
{
  global $PG_CONN;

  /** get bucket name */
  $sql = "SELECT bucket_name from bucket_def where bucket_pk = $bucket_pk;";
  $result = pg_query($PG_CONN, $sql);
  DBCheckResult($result, $sql, __FILE__, __LINE__);
  $row = pg_fetch_assoc($result);
  pg_free_result($result);

  print "For uploadtree ID $uploadtree_pk have bucket $row[bucket_name]:\n";
  $uploadtree_tablename = GetUploadtreeTableName($upload_pk);

  /* get the top of tree */
  $sql = "SELECT upload_fk, lft, rgt from uploadtree where uploadtree_pk='$uploadtree_pk';";
  $result = pg_query($PG_CONN, $sql);
  DBCheckResult($result, $sql, __FILE__, __LINE__);
  $toprow = pg_fetch_assoc($result);
  pg_free_result($result);

  /* loop through all the records in this tree */
  $sql = "select uploadtree_pk, ufile_name, lft, rgt from $uploadtree_tablename, bucket_file
    where upload_fk=$upload_pk
    and lft>'$toprow[lft]'  and rgt<'$toprow[rgt]'
    and ((ufile_mode & (1<<28)) = 0)  and ((ufile_mode & (1<<29)) = 0) and bucket_file.pfile_fk = $uploadtree_tablename.pfile_fk
    and bucket_fk = '$bucket_pk' and agent_fk = '$bucket_agent' and nomosagent_fk = '$nomos_agent'
    order by uploadtree_pk";
  $outerresult = pg_query($PG_CONN, $sql);

  /* Select each uploadtree row in this tree, write out text */
  while ($row = pg_fetch_assoc($outerresult))
  { 
    $filepatharray = Dir2Path($row['uploadtree_pk'], $uploadtree_tablename);
    $filepath = "";
    foreach($filepatharray as $uploadtreeRow)
    {
      if (!empty($filepath)) $filepath .= "/";
      $filepath .= $uploadtreeRow['ufile_name'];
    }
    $V = $filepath;
    print "$V";
    print "\n";
  } 
    pg_free_result($outerresult);
}

?>
